;
; Copyright 2005, 2006 University of Leiden.
;
; This file is part of MIA+EWS.
;
; MIA+EWS is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; MIA+EWS is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with MIA; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
PRO FringeLab__Define
;********************************************************
;#structure#
; FringeLabStruct
;#inheritance#
; NONE
;#description#
;internal variables for lab Fringe convenience object
;#structure_text#
fringe = {fringelab, $
INHERITS scannedFringe, $
INHERITS photoChop,  $
doPrint:0B, $
printFile:'', $
printUnit:0, $
labMaskFile:'', $
labFilterFile:'' $
}
;#end_structure#
;********************************************************
RETURN
END
PRO fringeLab::setPrint, doPrint
   if (self.printUnit NE 0) then begin
      FREE_LUN,self.printUnit
      self.printUnit = 0
   endif
   self.doPrint = doPrint
   self.printFile = '/tmp/EWS.'+STRTRIM(1000*randomu(seed,1),2)
   if (doPrint NE 0) then begin
      print,' printing into file ', self.printFile
      openw,self.printUnit,self.printFile,/GET_LUN
   endif
RETURN
END

PRO fringeLab::output,s1
   print,s1
   if (self.doPrint GT 0) then printf,self.printUnit, s1
RETURN
END

PRO fringeLab::print,printer=printer
   if (KEYWORD_SET(printer)) then pCom = ' lpr -P'+STRTRIM(printer,2)+' ' $
      else pCom = ' lpr '
   spawn,'a2ps '+self.printFile+' | pCom
RETURN
END
   
FUNCTION fringeLab::init, file, gui=gui, config=config, doPrint=doPrint
;********************************************************
;#function#
;fringeLab
;#description#
;convenience object storing scanned fringe object and other information
; supposedly easier to deal with than raw scanned fringe object
;#call#
;FL = OBJECT_NEW('fringeLab' ,maskfile)
;FL = OBJECT_NEW('fringeLab', maskdir, /gui)
;FL = OBJECT_NEW('fringeLab', configFile, /config)
;FL = OBJECT_NEW('fringeLab', /config)
;#inputs#
;\anArg{maskfile}{str}{mask file name}
;\anArg{gui}{flag}{Use IDL gui to find mask file}
;\anArg{config}{flag}{use ASCII configuration file for mask/filter files}
;#return#
;\anArg{-}{fringeLab object}{}
;#end_function#
;********************************************************
   self.printUnit = 0
   if (KEYWORD_SET(doPrint)) then self->setPrint,doPrint
   filterFile = ''
   if (KEYWORD_SET(config)) then begin
      if (N_PARAMS() EQ 1) then configFile = file $
         else configFile = GETENV('midiutil')+'/config.def'
      openr,unit,configFile,/GET_LUN
      maskfile = ''
      readf,unit, maskfile
      self.labMaskFile = maskFile
      readf,unit, maskfile  ; dispersed Mask file not used
      maskFile = self.labMaskFile
      readf,unit, filterFile
      sensFile = ''
      readf,unit, sensFile
      self.sensitivityFile = sensFile
      FREE_LUN, unit
   endif else if (NOT KEYWORD_SET(gui)) then $
      self.labMaskFile = maskfile $
   else begin
      if (N_PARAMS() LT 1) then maskDir = '.' else maskDir = maskFile
      f = DIALOG_PICKFILE(title='Pick Mask File', path=maskdir)
      self.labMaskFile = f
   endelse
   self->output,'MASK FILE IS   '+ self.labMaskFile
   self->output,'SENSITIVITY FILE IS   '+ self.sensitivityFile
   self.labFilterFile = filterFile
   if (filterFile NE '') then self->output,'FILTER FILE IS '+filterFile $
      else self->output,'FILTER FILE NOT YET DEFINED'
   if (1 NE self->photoChop::init(self.labMaskFile)) then RETURN,0
   if (1 NE self->scannedFringe::init(self.labMaskFile)) then RETURN,0
RETURN,1
END

PRO fringeLab::cleanup
   self->scannedFringe::cleanup
   self->photoChop::cleanup
RETURN
END

FUNCTION fringeObj, file, gui=gui, config=config
;********************************************************
;#function#
;fringeObj
;#description#
;alias for OBJ_NEW(fringeLab) for people who don't like OBJ_NEW syntax
;#call#
;z = fringeObj()
;z = fringeObj(maskdir)
;#inputs#
;\anArg{maskdir}{str}{directory where to start looking for mask files}
;\anArg{filterdir}{str}{directory where to start looking for filter files}
;#return#
;\anArg{-}{fringeLab Object}{}
;#end_function#
;********************************************************
   if (N_PARAMS() EQ 1) then RETURN, $
      OBJ_NEW('fringeLab', file , gui=gui, config=config)
   if (NOT KEYWORD_SET(gui)) AND (NOT KEYWORD_SET(config)) then config=1
   RETURN, OBJ_NEW('fringeLab', gui=gui, config=config)
END

PRO fringeLab::mask, maskfile,gui=gui
;********************************************************
;#procedure#
;mask
;#description#
;replace pixel mask file, optionally using gui
;#call#
;fringeLabObj->mask,maskfile
;fringeLabObj->mask,maskdir,/gui
;fringeLabObj->mask,/gui (default to current dir)
;#inputs#
;\anArg{maskfile}{str}{name of maskfile}
;\anArg{maskdir}{str}{name of dir to start file GUI in}
;\anArg{gui}{flag}{use gui to find file}
;#end_procedure#
;********************************************************
   if (KEYWORD_SET(gui)) then begin
      if (N_PARAMS() LT 1) then maskdir = '.' else maskdir=maskfile
      self.maskfile = DIALOG_PICKFILE(title='Pick Mask File', path=maskdir)
   endif else begin
      if (N_PARAMS() LT 1) then begin
         self->output,'No mask file specified'
         RETURN
      endif
      self.maskfile = maskfile
   endelse
   PTR_FREE,self.maskData
   self->output,'Selected Mask File '+self.maskfile
   if (self.maskFile NE '') then self.maskData = $
      PTR_NEW(oirGetData(self.maskFile))
RETURN
END

PRO fringeLab::getFilter, filterdir
;********************************************************
;#procedure#
;getFilter
;#description#
;use file GUI to select or replace spectral filter file
;#call#
;fringeObj->getFilter
;fringeObj->getFilter, filterdir
;z = fringeObj(maskdir)
;#inputs#
;\anArg{filterdir}{str}{directory where to start looking for filter files}
;#end_procedure#
;********************************************************
   if (N_PARAMS() LT 1) then filterdir = '.'
   f = DIALOG_PICKFILE(title='Pick filter File', path=filterdir)
   self.labfilterfile = f
   print,'FILTER FILE IS   ', self.filterfile
   self->scannedFringe::filter,f
RETURN
END

PRO fringeLab::getFringeData, input, compressLast=compressLast, last=last,$
    gui=gui
;********************************************************
;#procedure#
;getFringeData
;#description#
;fetch fringe data for this object from disk, mask it and compress it
;and store internally.  If mask is not defined I will insist here
;#call#
;fringeObj->getFringeData, datafile
;fringeObj->getFringeData, dataDir,/gui
;fringeObj->getFringeData, /gui
;fringeObj->getData, /last
;fringeObj->getData, /compressLast
;#inputs#
;\anArg{datafile}{string}{datafile is /gui, /last and /comp not specified}
;\anArg{dataDir}{string}{directory to look for data; if /gui etc.default is "."}
;\anArg{gui}{flag}{use gui to find data}
;\anArg{last}{flag}{use most recent data file in directory}
;\anArg{last}{compressLast}{use most recent set of multiple files in directory}
;#end_procedure#
;********************************************************
; cleanup existing data
; decide where to get data
   IF (KEYWORD_SET(compressLast)) THEN BEGIN
      IF (N_PARAMS() EQ 1) then dir = input else dir = '.'
      cd,current=currentdir
      cd,dir
      files = gfu()
      fits = OBJ_NEW('fitsFileList', files)
      fits->midiCompress
      files = fits->files()
      print,'Compressing and storing ',files[0]
      dataFile = files[0]
      cd,currentdir
   ENDIF ELSE IF (KEYWORD_SET(last)) THEN BEGIN
      IF (N_PARAMS() EQ 1) then dir = input else dir = '.'
      cd,current=currentdir
      cd,dir
      files = gfu(last)
      print,'Compressing and storing ',last
      dataFile = last
      cd,currentdir
   ENDIF ELSE IF (KEYWORD_SET(gui)) THEN BEGIN
      IF (N_PARAMS() EQ 1) then dir = input else dir = '.'
      cd,current=currentdir
      cd,dir
      files = midigui()
      print,'Compressing and storing ',files[0]
      cd,currentdir
      dataFile = files[0]
   ENDIF ELSE dataFile = input
      print,'Getting Interferometric Data  '
      self->getFileData, dataFile
      self->getFitsData, dataFile
; FILTER
   print,' '
   print,'filtering data'
   self->scannedFringe::filter,self.labFilterFile
   self->kData
RETURN
END

PRO fringeLab::getPhotoData, file, gui=gui
   self->photoChop::getData, file, gui=gui
RETURN
END

FUNCTION fringeLab::maskData
RETURN,oirGetData(self.labMaskFile)
END

FUNCTION fringeLab::waveNo
;********************************************************
;#function#
;waveNo	
;#description#
;returns array of waveNumbers (reciprocal wavelengths/microns)
; corresponding to power spectra abscissa
;#call#
;spFreq = labObject->outOpd()
;#inputs#
;#return#
;\anArg{-}{fltarr}{OPD array}
;#end_function#
;********************************************************
RETURN,self->scannedFringe::kGrid()/(2*!pi)
END

PRO fringeLab::usePhotoChop
   self.photo=REFORM(self.maskFlux[2,*])
RETURN
END

PRO fringeLab::resetLabMask, maskFile
   self.labMaskFile = maskFile
RETURN
END

FUNCTION estimateVis, file, files=files, $
   save=save, config=config, newMask=newMask, $
   aData = aData, bData=bData, abData=abData, maskData=maskData
;
   if (N_PARAMS() GT 0) then z=obj_new('fringelab',file,/config)$
      else z=obj_new('fringelab',/config)
   if (N_ELEMENTS(files) GT 0) then d = files $
      else d=midigui()
   if(N_ELEMENTS(d) NE 3) then begin
      print,'I WANT 3 and ONLY 3 files' 
      RETURN,0
   endif
   fileList = REPLICATE(-1,3)
   for i=0,2 do begin
      midiGetIns,d[i],shutter=shutter
      shutter = STRTRIM(shutter,2)
      if (shutter EQ 'AOPEN') then fileList[0]=i $
      else if (shutter EQ 'BOPEN') then fileList[1] = i $
      else if (shutter EQ 'ABOPEN') then fileList[2] = i
   endfor
   if (TOTAL (fileList EQ (-1))) NE 0 then begin
      print,' I cant find two photo and one interf file'
      RETURN,0
   endif
   print,'PHOTO A file(s) ',d[fileList[0]]
   print,'PHOTO B file(s) ',d[fileList[1]]
   print,'FOURIER INTERF  file(s) ',d[fileList[2]]
   midigetINS,d[fileList[2]],object=object
   print,' OBJECT ',object
   z->getfringedata,d[fileList[2]]
   if (KEYWORD_SET(newMask)) then begin
      z->newMask,'/tmp/maskFile.fits'
      z->resetPhotoMask, '/tmp/maskFile.fits'
      z->resetLabMask,   '/tmp/maskFile.fits'
   endif
   maskdata = z->maskData()
   z->getphotodata,[d[fileList[0]], d[fileList[1]]]
   z->photoData, aData=aData, bData=bData, abData=abData
   z->usePhotoChop 
   z->set,/doPhoto
   ma=z->maxamp()
   z->autoselect
   plot,ma,yr=[0,1.2]
   s=z->select()
   oplot,s,thick=2
   print,' used ',TOTAL(s),' points out of ',N_ELEMENTS(ma)
   print,'VISIBILITIES:'
   print,z->visStats(),format='("mean vis ",f8.3," rms of sample ",f8.3," rms of mean", f8.3)'
   z->set,doPhoto=0
   print,'CORRELATED FLUXES:'
   print,z->visStats(),format='("mean vis ",f8.0," rms of sample ",f8.0," rms of mean", f8.0)'
   if (NOT KEYWORD_SET(save)) then begin
      OBJ_DESTROY,z
      z = OBJ_NEW()
   endif
RETURN,z
END
