;
; Copyright 2005, 2006 University of Leiden.
;
; This file is part of MIA+EWS.
;
; MIA+EWS is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; MIA+EWS is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with MIA; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
PRO photoChop__define
p={ photoChop, $
photoMaskFile:'', $
maskData:PTR_NEW(), $
AChopFile:'', $
BChopFile:'', $
AChopData:PTR_NEW(), $
BChopData:PTR_NEW(), $
ABChopData:PTR_NEW(), $
maskFlux:fltarr(3,2), $ ;A,B,AB for 1,2 $
totalFlux:fltarr(3,2) $ ;A,B,AB for 1,2 $
}
RETURN
END

FUNCTION photoChop::init, maskFile, gui=gui
;
   if (NOT KEYWORD_SET(gui)) then begin
      self.photoMaskFile = maskfile
      self.maskData = PTR_NEW(oirGetData(maskFile))
      RETURN,1
   endif
   if (N_PARAMS() LT 1) then maskdir = '.' else maskdir=maskFile
   f = DIALOG_PICKFILE(title='Pick Mask File', path=maskdir)
   self.photoMaskFile = f
   if (f NE '') then self.maskData = PTR_NEW(oirGetData(self.photoMaskFile))
RETURN,1
END

PRO photoChop::cleanup
   PTR_FREE,self.AChopData
   PTR_FREE,self.BChopData
   PTR_FREE,self.ABChopData
RETURN
END

PRO photoChop::getData, file, gui=gui
;********************************************************
;#procedure#
;getData
;#description#
;get files with photometric chopped data
;#call#
;chopObj->getData, fileNames
;chopObj->getData, /gui
;chopObj->getData, dir, /gui
;#inputs#
;\anArg{file}{strarr[2]}{file names of A and B photometry data}
;\anArg{file}{strarr}{directory for file gui}
;\anArg{gui}{flag}{use gui to find files}
;#end_procedure#
;********************************************************
   if (KEYWORD_SET(gui)) then begin
      if (N_PARAMS() LT 1) then file = '.'
      photoFiles = midigui(dir=file)
   endif else photoFiles = file
   if (N_ELEMENTS(photoFiles) NE 2) then begin
      print,'I need two files, A and B to do chopping photometry'
      RETURN
   endif
   self.AChopFile = photoFiles[0]
   self.BChopFile = photoFiles[1]
   PTR_FREE, self.AChopData
   PTR_FREE, self.BChopData
   PTR_FREE, self.ABChopData
   print,' Reading Chop Photometry Data from ',self.AChopFile
   print,' Reading Chop Photometry Data from ',self.BChopFile
   data = midiChopImage(self.AChopFile)
   midigetINS, self.AChopFile, dit=dit ; exposure time
   data[0].DATA1 = (data[0].DATA1 - data[1].DATA1)/dit
   data[0].DATA2 = (data[0].DATA2 - data[1].DATA2)/dit
   self.AChopData = PTR_NEW(data[0])
   data = midiChopImage(self.BChopFile)
   data[0].DATA1 = (data[0].DATA1 - data[1].DATA1)/dit
   data[0].DATA2 = (data[0].DATA2 - data[1].DATA2)/dit
   self.BChopData = PTR_NEW(data[0])
   self.ABChopData = PTR_NEW(data[0])
   (*self.ABChopData).DATA1 = (*self.AChopData).DATA1 * (*self.BChopData).DATA1 
   (*self.ABChopData).DATA2 = (*self.AChopData).DATA2 * (*self.BChopData).DATA2
   if (NOT PTR_VALID(self.maskData)) then print,'NO mask data yet available' else begin
      self.maskFlux[0,0] = TOTAL((*self.maskData).DATA1 * (*self.AChopData).DATA1)
      self.maskFlux[1,0] = TOTAL((*self.maskData).DATA1 * (*self.BChopData).DATA1)
      self.maskFlux[2,0] = TOTAL((*self.maskData).DATA1 * SQRT((*self.ABChopData).DATA1> 0.))
      self.maskFlux[0,1] = TOTAL((*self.maskData).DATA2 * (*self.AChopData).DATA2)
      self.maskFlux[1,1] = TOTAL((*self.maskData).DATA2 * (*self.BChopData).DATA2)
      self.maskFlux[2,1] = TOTAL((*self.maskData).DATA2 * SQRT((*self.ABChopData).DATA2 > 0.))
   endelse
   self.totalFlux[0,0] = TOTAL((*self.AChopData).DATA1)
   self.totalFlux[1,0] = TOTAL((*self.BChopData).DATA1)
   self.totalFlux[2,0] = TOTAL(SQRT((*self.ABChopData).DATA1> 0.))
   self.totalFlux[0,1] = TOTAL((*self.AChopData).DATA2)
   self.totalFlux[1,1] = TOTAL((*self.BChopData).DATA2)
   self.totalFlux[2,1] = TOTAL(SQRT((*self.ABChopData).DATA2 > 0.))
   PRINT,'Masked Fluxes', self.maskFlux, $
      format="(a,/,'    A        B        AB   ',/,2(3e10.3,/))"
   PRINT,'Total Fluxes', self.totalFlux, $
      format="(a,/,'    A        B        AB   ',/,2(3e10.3,/))"
RETURN
END

FUNCTION photoChop::deSkyImage, image, mask, maskFlux, totalFlux, sky, skyErr, $
   maxR=maxR, tol=tol
;********************************************************
;#function#
;deSkyImage
;#description#
;estimate sky from 1 image and remove it
;#call#
;sky = fringeLabObj->findSky(image, mask, maskFlux, totalFlux, sky, skyErr, maxR=maxR, tol=tol)
;#inputs#
;\anArg{image}{fltarr}{input image}
;\anArg{mask}{fltarr}{array of same dim as image, weights per pixel}
;\anArg{maxR}{flt}{look out to maxR pixels; if haven't reached sky yet; sky=0}
;\anArg{tol}{flt}{look out until fract. change in sky contrib. to flux < tol ;def=.01}
;#outputs#
;\anArg{maskFlux}{flt}{estimated flux under mask}
;\anArg{totalFlux}{flt}{estimated total flux to effective maxR}
;\anArg{sky}{flt}{estimated sky per pixel}
;\anArg{skyerr}{flt}{estimated rms sky per pixel}
;#returns#
;\anArg{-}{fltarr}{image - sky}
;#end_function#
;********************************************************
   if (NOT KEYWORD_SET(tol)) then tol = 0.01
   if (NOT KEYWORD_SET(maxR)) then maxR = 15
; dimensions of image array and pos of max.
   sImage = SIZE(image, /dimension)
   nX = sImage[0]
   nY = sImage[1]
   rr = 0.*image
   m = MAX(image, i)
   x0 = i mod nX
   y0 = i / nX
; array of radius from maximum in pixels
   for i=0,nY-1 do rr(*,i) = (findgen(nX)-x0)^2
   for i=0,nX-1 do rr(i,*) = rr(i,*)+(findgen(nY)-y0)^2
   rr = SQRT(rr)
   mR = maxR < (x0) < (nX-x0) < y0 < (nY-y0) ; max usable radius
; arrays of mean and rms sky estimate in circular aprs.
   skyFlux  = fltarr(mR)
   skyFlux2 = fltarr(mR)
   for i=1, mR-1 do begin 
      s=WHERE((rr ge (i-1)) AND (rr le (i+1)))
      m = moment(image[s])
      skyFlux[i]  = m[0]
      skyFlux2[i] = m[1]
   endfor
; derivative of change
  dS = skyFlux - shift(skyFlux,1)
  dS[0] = -1.e9  ; large negative number
  flux0 = TOTAL(mask*image)
  tMask = TOTAL(mask)
; where change in sky estimate is < tol * flux
  sMin = MIN(WHERE(dS GT (-tol * ABS(flux0/tMask)), nWhere))
  if (nWhere EQ 0) then sMin = mR
  sky = skyFlux[sMin]
  skyErr = SQRT(skyFlux2[sMin])
  output = image - sky
  maskFlux = TOTAL(mask * output)
  s=WHERE(rr le sMin)
  totalFlux = TOTAL(output(s))
RETURN, image - sky
END 

PRO photoChop::deSky, tol = tol
;********************************************************
;#procedure#
;deSky
;#description#
;remove sky estimates from internal photom. data
;#call#
;fringeLabObj->deSky
;#end_procedure#
;********************************************************
   if (NOT PTR_VALID(self.maskData)) then begin
      print,'I need mask file for my sky removal technique'
      RETURN
   endif
   if (PTR_VALID(self.AChopData)) then begin 
      (*self.AChopData).DATA1 = $
         self->deSkyImage((*self.AChopData).DATA1, (*self.maskData).DATA1, m,t,tol=tol)
      self.maskFlux[0,0]  = m
      self.totalFlux[0,0] = t
   endif
   if (PTR_VALID(self.AChopData)) then begin 
      (*self.AChopData).DATA2= $
         self->deSkyImage((*self.AChopData).DATA2, (*self.maskData).DATA2, m,t,tol=tol)
      self.maskFlux[0,1]  = m
      self.totalFlux[0,1] = t
   endif
   if (PTR_VALID(self.BChopData)) then begin 
      (*self.BChopData).DATA1 = $
         self->deSkyImage((*self.BChopData).DATA1, (*self.maskData).DATA1,m,t,tol=tol)
      self.maskFlux[1,0]  = m
      self.totalFlux[1,0] = t
   endif
   if (PTR_VALID(self.BChopData)) then begin 
      (*self.BChopData).DATA2= $
         self->deSkyImage((*self.BChopData).DATA2, (*self.maskData).DATA2,m,t,tol=tol)
      self.maskFlux[1,1]  = m
      self.totalFlux[1,1] = t
   endif
   if (PTR_VALID(self.ABChopData)) then (*self.ABChopData).DATA1 = $
      (*self.AChopData).DATA1 * (*self.BChopData).DATA1 
   if (PTR_VALID(self.ABChopData)) then (*self.ABChopData).DATA2 = $
      (*self.AChopData).DATA2 * (*self.BChopData).DATA2 
   self.maskFlux[2,0] = TOTAL((*self.maskData).DATA1 * SQRT((*self.ABChopData).DATA1>0))
   self.maskFlux[2,1] = TOTAL((*self.maskData).DATA2 * SQRT((*self.ABChopData).DATA2>0))
   self.totalFlux[2,0] = TOTAL(SQRT((*self.ABChopData).DATA1 > 0))
   self.totalFlux[2,1] = TOTAL(SQRT((*self.ABChopData).DATA2 > 0))
   PRINT,'Masked fluxes :',self.maskFlux,$
      format="(a,/,'    A        B        AB   ',/,2(3e10.3,/))"
   PRINT,'Total  fluxes :',self.totalFlux, $
      format="(a,/,'    A        B        AB   ',/,2(3e10.3,/))"
RETURN
END 

PRO photoChop::photoData, aData=aData, bData=bData, abData=abData, $
   mask=mask, total=total
;********************************************************
;#procedure#
;photoData
;#description#
;return stored photometry data
;#call#
;fringeLabObj->photoData, aData, bData
;#outputs#
;\anArg{aData}{fltarr}{Image of A(source)-A(sky)}
;\anArg{bData}{fltarr}{Image of B(source)-A(sky)}
;\anArg{bData}{fltarr}{aData*bData}
;\anArg{fluxes}{fltarr}{integrated fluxes (*mask) A,B,AB for channels 1,2}
;#end_procedure#
;********************************************************
   if (PTR_VALID(self.AChopData)) then aData = *self.AChopData else RETURN
   if (PTR_VALID(self.BChopData)) then bData = *self.BChopData else RETURN
   if (PTR_VALID(self.ABChopData)) then abData = *self.ABChopData else RETURN
   mask = self.maskFlux
   total = self.totalFlux
RETURN
END

PRO photoChop::resetPhotoMask, maskFile
;
   self.photoMaskFile = maskFile
   PTR_FREE,self.maskData
   if (maskFile NE '') then self.maskData = PTR_NEW(oirGetData(self.photoMaskFile))
RETURN
END
